//--------------------------------------------------------------
// File     : stm32_ub_graphic.c
// Datum    : 16.03.2013
// Version  : 1.5
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : STM32_UB_LCD_ST7783
// Funktion : Grafik-LCD Funktionen
//            (Punkte, Linien, Kreise)
//            und Bilder aus dem Flash anzeigen
//
// Formate  : 1. 16bit-Images (RGB656)
//            2. Windows-BMP (24bpp, unkompremiert)  
//            3. Windows-JPG
//
// Speed    : 1. Image 240x320 (16bpp) von Flash ca. 18ms
//            2. BMP 240x320 (24bpp) von Flash ca. 42 ms
//            3. JPG 240x320 von Flash ca. 566 ms
//
// Hinweis  : zum konvertieren der Bilder in das notwendige
//            Format, gibt es die PC-Programme :
//            "ImageGenerator_UB.exe" und "FileConverter_UB.exe"
//--------------------------------------------------------------

//--------------------------------------------------------------
// Quelle vom JPG-Encoder :
// http://en.pudn.com/downloads263/sourcecode/
//        embed/detail1210390_en.html
// Version 1.4 (2009.04.21)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_graphic.h"




//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
int16_t P_Graphic_sgn(int16_t x);
#if GRAPHIC_USE_BMP==1
GRAPHIC_ERR_t P_Graphic_Check_BMPHeader(UB_Picture *bmp);
uint32_t P_Graphic_4Bytes(UB_Picture *bmp, uint32_t start);
uint16_t P_Graphic_2Bytes(UB_Picture *bmp, uint32_t start);
#endif
#if GRAPHIC_USE_JPG==1
int InitTag(void);
void InitTable(void);
int Decode(void);
void GetYUV(short flag);
void StoreBuffer(void);
int DecodeMCUBlock(void);
int HufBlock(BYTE dchufindex,BYTE achufindex);
int DecodeElement(void);
void IQtIZzMCUComponent(short flag);
void IQtIZzBlock(short  *s ,short * d,short flag);
void Fast_IDCT(int * block);
BYTE ReadByte(void);
void Initialize_Fast_IDCT(void);
void idctrow(int * blk);
void idctcol(int * blk); 
#endif


//--------------------------------------------------------------
// Globale Variabeln fr JPG
//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
const int GRAPHIC_JPG_ZZ[8][8]={
  {0,1,5,6,14,15,27,28},
  {2,4,7,13,16,26,29,42},
  {3,8,12,17,25,30,41,43},
  {9,11,18,24,37,40,44,53},
  {10,19,23,32,39,45,52,54},
  {20,22,33,38,46,51,55,60},
  {21,34,37,47,50,56,59,61},
  {35,36,48,49,57,58,62,63}
};
#endif

#if GRAPHIC_USE_JPG==1
const BYTE GRAPHIC_JPG_AND[9]={
  0,1,3,7,0xf,0x1f,0x3f,0x7f,0xff
};
#endif


//--------------------------------------------------------------
// Zeichnet ein Pixel mit einer Farbe an x,y Position
//--------------------------------------------------------------
void UB_Graphic_DrawPixel(int16_t xpos, int16_t ypos, uint16_t color)
{
  // Cursor setzen
  UB_LCD_SetCursor2Draw(xpos,ypos);
  // Pixel zeichnen
  LCD_RAM = color;
}


//--------------------------------------------------------------
// Zeichnet eine Linie mit einer Farbe
// von x1,y1 nach x2,y2
// (benutzt wird der Bresenham-Algorithmus)





//--------------------------------------------------------------
void UB_Graphic_DrawLine(int16_t y1, int16_t x1, int16_t y2, int16_t x2, uint16_t color)
{
  int16_t x, y, t, dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;
 
  // Entfernung in beiden Dimensionen berechnen
  dx = x2 - x1;
  dy = y2 - y1;
 
  // Vorzeichen des Inkrements bestimmen
  incx = P_Graphic_sgn(dx);
  incy = P_Graphic_sgn(dy);
  if(dx<0) dx = -dx;
  if(dy<0) dy = -dy;
 
  // feststellen, welche Entfernung grer ist
  if (dx>dy) {
    // x ist schnelle Richtung 
    pdx=incx; pdy=0;    // pd. ist Parallelschritt
    ddx=incx; ddy=incy; // dd. ist Diagonalschritt
    es =dy;   el =dx;   // Fehlerschritte schnell, langsam
  }
  else {
    // y ist schnelle Richtung
    pdx=0;    pdy=incy; // pd. ist Parallelschritt
    ddx=incx; ddy=incy; // dd. ist Diagonalschritt
    es =dx;   el =dy;   // Fehlerschritte schnell, langsam
  }
 
  // Initialisierungen vor Schleifenbeginn
  x = x1;
  y = y1;
  err = (el>>1);
  UB_Graphic_DrawPixel(x,y,color);
 
  // Pixel berechnen
  for(t=0; t<el; ++t) // t zaehlt die Pixel, el ist auch Anzahl
  {
    // Aktualisierung Fehlerterm
    err -= es; 
    if(err<0) {
      // Fehlerterm wieder positiv (>=0) machen
      err += el;
      // Schritt in langsame Richtung, Diagonalschritt
      x += ddx;
      y += ddy;
    }
    else {
      // Schritt in schnelle Richtung, Parallelschritt
      x += pdx;
      y += pdy;
    }
    UB_Graphic_DrawPixel(x,y,color);
  }
}

//--------------------------------------------------------------
// Zeichnet einen Kreis mit einer Farbe
// Mittelpunkt = x1,y1
// Radius = radius
// (benutzt wird der Bresenham-Algorithmus)
//--------------------------------------------------------------
void UB_Graphic_DrawCircle(int16_t x0, int16_t y0, int16_t radius, uint16_t color)
{
  int16_t f=1-radius, ddF_x=0, ddF_y=-2*radius, x=0, y=radius;

  UB_Graphic_DrawPixel(x0, y0 + radius, color);
  UB_Graphic_DrawPixel(x0, y0 - radius, color);
  UB_Graphic_DrawPixel(x0 + radius, y0, color);
  UB_Graphic_DrawPixel(x0 - radius, y0, color);

  while(x < y)
  {
    if(f >= 0)
    {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x + 1;

    UB_Graphic_DrawPixel(x0 + x, y0 + y, color);
    UB_Graphic_DrawPixel(x0 - x, y0 + y, color);
    UB_Graphic_DrawPixel(x0 + x, y0 - y, color);
    UB_Graphic_DrawPixel(x0 - x, y0 - y, color);
    UB_Graphic_DrawPixel(x0 + y, y0 + x, color);
    UB_Graphic_DrawPixel(x0 - y, y0 + x, color);
    UB_Graphic_DrawPixel(x0 + y, y0 - x, color);
    UB_Graphic_DrawPixel(x0 - y, y0 - x, color);
  }
}


#if GRAPHIC_USE_IMG==1
//--------------------------------------------------------------
// Zeichnet ein 16bit Image (aus dem Flash)
// PORTRAIT  : linke obere Ecke = xpos,ypos
// LANDSCAPE : linke untere Ecke = xpos,ypos
// -> Image muss mit &-Operator uebergeben werden
// Return_wert :
//     GRAPHIC_OK , Bild gezeichnet
//  != GRAPHIC_OK , fehler beim zeichnen
// Konvertierungs-Programm : "ImageGenerator_UB.exe"
//--------------------------------------------------------------
GRAPHIC_ERR_t UB_Graphic_DrawImage(UB_Image *img, int16_t xpos, int16_t ypos)
{
  GRAPHIC_ERR_t ret_wert=GRAPHIC_OK;
  uint16_t xn,yn;
  const uint16_t *wert;

  wert=&img->table[0];  

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    if(img->width>LCD_MAXX) return(GRAPHIC_WIDTH_ERR);
    if(img->height>LCD_MAXY) return(GRAPHIC_HEIGHT_ERR);    

    for(yn = 0; yn < img->height; yn++) {
      // Cursor setzen
      UB_LCD_SetCursor2Draw(xpos,yn+ypos);
      for(xn = 0; xn < img->width; xn++) {
        LCD_RAM = wert[yn*img->width+xn];
      }
    }
  }
  else {
    if(img->width>LCD_MAXY) return(GRAPHIC_WIDTH_ERR);
    if(img->height>LCD_MAXX) return(GRAPHIC_HEIGHT_ERR);    

    for(yn = 0; yn < img->height; yn++) {
      // Cursor setzen
      UB_LCD_SetCursor2Draw(img->height+xpos-yn-1,ypos);
      for(xn = 0; xn < img->width; xn++) {
        LCD_RAM = wert[yn*img->width+xn];
      }
    }
  }

  return(ret_wert);
}
#endif


#if GRAPHIC_USE_IMG==1
//--------------------------------------------------------------
// Zeichnet ein 16bit Image (aus dem Flash)
// die definierte Colorfarbe wird nicht gezeichnet
// und ist damit Transparent
// PORTRAIT  : linke obere Ecke = xpos,ypos
// LANDSCAPE : linke untere Ecke = xpos,ypos
// -> Image muss mit &-Operator uebergeben werden
// Return_wert :
//     GRAPHIC_OK , Bild gezeichnet
//  != GRAPHIC_OK , fehler beim zeichnen
// Konvertierungs-Programm : "ImageGenerator_UB.exe"
//--------------------------------------------------------------
GRAPHIC_ERR_t UB_Graphic_DrawImageTransparent(UB_Image *img, int16_t xpos, int16_t ypos, uint16_t color)
{
  GRAPHIC_ERR_t ret_wert=GRAPHIC_OK;
  uint16_t xn,yn,img_color;
  const uint16_t *wert;

  wert=&img->table[0];

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    if(img->width>LCD_MAXX) return(GRAPHIC_WIDTH_ERR);
    if(img->height>LCD_MAXY) return(GRAPHIC_HEIGHT_ERR);

    for(yn = 0; yn < img->height; yn++) {
      // Cursor setzen
      UB_LCD_SetCursor2Draw(xpos,yn+ypos);
      for(xn = 0; xn < img->width; xn++) {
        img_color = wert[yn*img->width+xn];
        if(img_color!=color) {
          LCD_RAM = img_color;
        }
        else {
          UB_LCD_SetCursor2Draw(xn+xpos,yn+ypos);
        }
      }
    }
  }
  else {
    if(img->width>LCD_MAXY) return(GRAPHIC_WIDTH_ERR);
    if(img->height>LCD_MAXX) return(GRAPHIC_HEIGHT_ERR);

    for(yn = 0; yn < img->height; yn++) {
      // Cursor setzen
      UB_LCD_SetCursor2Draw(img->height+xpos-yn,ypos);
      for(xn = 0; xn < img->width; xn++) {
        img_color = wert[yn*img->width+xn];
        if(img_color!=color) {
          LCD_RAM = img_color;
        }
        else {
          UB_LCD_SetCursor2Draw(img->height+xpos-yn-1,ypos+xn);
        }
      }
    }
  }

  return(ret_wert);
}
#endif


#if GRAPHIC_USE_BMP==1
//--------------------------------------------------------------
// zeichnet ein Windows-BMP-File (aus dem Flash)
// PORTRAIT  : linke obere Ecke = xpos,ypos
// LANDSCAPE : linke untere Ecke = xpos,ypos
// -> BMP muss mit &-Operator uebergeben werden
// Return_wert :
//     GRAPHIC_OK , Bild gezeichnet
//  != GRAPHIC_OK , fehler beim zeichnen
// Konvertierungs-Programm : "FileConverter_UB.exe"
//--------------------------------------------------------------
GRAPHIC_ERR_t UB_Graphic_DrawBmp(UB_Picture *bmp, uint16_t xpos, uint16_t ypos)
{
  GRAPHIC_ERR_t ret_wert=GRAPHIC_OK;
  uint8_t blue, green, red;
  uint16_t xn,yn;
  const uint8_t *wert;

  wert=&bmp->table[0];

  // BMP-Header prfen
  ret_wert=P_Graphic_Check_BMPHeader(bmp);

  if(ret_wert==GRAPHIC_OK)  {

    if(LCD_DISPLAY_MODE==PORTRAIT) {
      for(yn = 0; yn < GRAPHIC_BMP.height; yn++) {
        UB_LCD_SetCursor2Draw(xpos,(GRAPHIC_BMP.height-yn)+ypos-1);
        for(xn = 0; xn < GRAPHIC_BMP.width; xn++) {
          blue=(wert[GRAPHIC_BMP.akt_ptr]&0xF8);    // 5bit
          green=(wert[GRAPHIC_BMP.akt_ptr+1]&0xFC); // 6bit
          red=(wert[GRAPHIC_BMP.akt_ptr+2]&0xF8);   // 5bit
          GRAPHIC_BMP.akt_ptr+=3;

          // Pixel auf Display zeichnen
          LCD_RAM = ((red<<8) | (green<<3) | (blue>>3));
        }
        GRAPHIC_BMP.akt_ptr+=GRAPHIC_BMP.spacer;
      }
    }
    else {
      for(yn = 0; yn < GRAPHIC_BMP.height; yn++) {
        UB_LCD_SetCursor2Draw(yn+xpos,ypos);
         for(xn = 0; xn < GRAPHIC_BMP.width; xn++) {
           blue=(wert[GRAPHIC_BMP.akt_ptr]&0xF8);    // 5bit
           green=(wert[GRAPHIC_BMP.akt_ptr+1]&0xFC); // 6bit
           red=(wert[GRAPHIC_BMP.akt_ptr+2]&0xF8);   // 5bit
           GRAPHIC_BMP.akt_ptr+=3;

           // Pixel auf Display zeichnen
           LCD_RAM = ((red<<8) | (green<<3) | (blue>>3));
        }
        GRAPHIC_BMP.akt_ptr+=GRAPHIC_BMP.spacer;
      }
    }
  }

  return(ret_wert);
}
#endif


#if GRAPHIC_USE_JPG==1
//--------------------------------------------------------------
// zeichnet ein Windows-JPG-File (aus dem Flash)
// PORTRAIT  : linke obere Ecke = xpos,ypos
// LANDSCAPE : linke untere Ecke = xpos,ypos
// -> JPG muss mit &-Operator uebergeben werden
// Return_wert :
//     GRAPHIC_OK , Bild gezeichnet
//  != GRAPHIC_OK , fehler beim zeichnen
// Konvertierungs-Programm : "FileConverter_UB.exe"
//--------------------------------------------------------------
GRAPHIC_ERR_t UB_Graphic_DrawJpg(UB_Picture *jpg, uint16_t xpos, uint16_t ypos)
{    
  GRAPHIC_ERR_t ret_wert=GRAPHIC_OK;
  const uint8_t *hJpegBuf;
  int funcret;

  GRAPHIC_JPG.CurX=xpos;
  GRAPHIC_JPG.CurY=ypos;
 
  hJpegBuf=&jpg->table[0]; 
  GRAPHIC_JPG.lpJpegBuf=(unsigned char*)hJpegBuf;  
  InitTable();  
  if((funcret=InitTag())!=FUNC_OK) { 	 
    return(GRAPHIC_FILE_ERR);
  }

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    if(GRAPHIC_JPG.ImgWidth>LCD_MAXX) return(GRAPHIC_WIDTH_ERR);
    if(GRAPHIC_JPG.ImgHeight>LCD_MAXY) return(GRAPHIC_HEIGHT_ERR);
  }
  else {
    if(GRAPHIC_JPG.ImgWidth>LCD_MAXY) return(GRAPHIC_WIDTH_ERR);
    if(GRAPHIC_JPG.ImgHeight>LCD_MAXX) return(GRAPHIC_HEIGHT_ERR);
  }

  if((GRAPHIC_JPG.SampRate_Y_H==0)||(GRAPHIC_JPG.SampRate_Y_V==0)) { 
    return(GRAPHIC_FILE_ERR);
  }	   

  funcret=Decode();
  if(funcret!=FUNC_OK) {   
    return(GRAPHIC_FILE_ERR);
  }

  return(ret_wert);
}
#endif



//--------------------------------------------------------------
// interne Funktion
// Header vom Windows-BMP-File prfen
//--------------------------------------------------------------
#if GRAPHIC_USE_BMP==1
GRAPHIC_ERR_t P_Graphic_Check_BMPHeader(UB_Picture *bmp)
{
  GRAPHIC_ERR_t ret_wert=GRAPHIC_SIZE_ERR;
  uint32_t temp;
  const uint8_t *wert;

  wert=&bmp->table[0];

  // Filesize bei BMP muss > 54 Bytes sein
  if(bmp->size<54) return(GRAPHIC_SIZE_ERR);
  // Check auf BMP-ID : 'BM'
  if((wert[0]!='B') || (wert[1]!='M')) return(GRAPHIC_ID_ERR);
  GRAPHIC_BMP.offset=P_Graphic_4Bytes(bmp,10);
  // Check auf Headersize : 40d
  temp=P_Graphic_4Bytes(bmp,14);
  if(temp!=40) return(GRAPHIC_HEAD_ERR);
  // Check auf Breite und Hoehe
  GRAPHIC_BMP.width=P_Graphic_4Bytes(bmp,18);
  GRAPHIC_BMP.height=P_Graphic_4Bytes(bmp,22);
  if(LCD_DISPLAY_MODE==PORTRAIT) {
    if(GRAPHIC_BMP.width>LCD_MAXX) return(GRAPHIC_WIDTH_ERR);
    if(GRAPHIC_BMP.height>LCD_MAXY) return(GRAPHIC_HEIGHT_ERR);
  }
  else {
    if(GRAPHIC_BMP.width>LCD_MAXY) return(GRAPHIC_WIDTH_ERR);
    if(GRAPHIC_BMP.height>LCD_MAXX) return(GRAPHIC_HEIGHT_ERR);
  }
  // Check auf BitPerPixel : 24d
  GRAPHIC_BMP.bpp=P_Graphic_2Bytes(bmp,28);
  if(GRAPHIC_BMP.bpp!=24) return(GRAPHIC_BPP_ERR);
  // Check auf Kompression : keine
  GRAPHIC_BMP.compr=P_Graphic_4Bytes(bmp,30);
  if(GRAPHIC_BMP.compr!=0) return(GRAPHIC_COMPR_ERR);
  // Variabeln setzen
  GRAPHIC_BMP.spacer=(GRAPHIC_BMP.width & 0x03);
  GRAPHIC_BMP.akt_ptr=GRAPHIC_BMP.offset;
  ret_wert=GRAPHIC_OK;

  return(ret_wert);
}
#endif


//--------------------------------------------------------------
// interne Funktion
// DWORD (4Bytes) vom Puffer ab start auslesen
//--------------------------------------------------------------
#if GRAPHIC_USE_BMP==1
uint32_t P_Graphic_4Bytes(UB_Picture *bmp, uint32_t start)
{
  uint32_t ret_wert=0;
  const uint8_t *wert;

  wert=&bmp->table[0];

  ret_wert|=(wert[start]);
  ret_wert|=(wert[start+1]<<8);
  ret_wert|=(wert[start+2]<<16);
  ret_wert|=(wert[start+3]<<24);

  return(ret_wert);
}
#endif


//--------------------------------------------------------------
// interne Funktion
// WORD (2Bytes) vom Puffer ab start auslesen
//--------------------------------------------------------------
#if GRAPHIC_USE_BMP==1
uint16_t P_Graphic_2Bytes(UB_Picture *bmp, uint32_t start)
{
  uint16_t ret_wert=0;
  const uint8_t *wert;

  wert=&bmp->table[0];

  ret_wert|=(wert[start]);
  ret_wert|=(wert[start+1]<<8);

  return(ret_wert);
}
#endif


//--------------------------------------------------------------
// interne Funktion
// Signum funktion
//  Return_Wert
//    1 ,wenn x > 0
//    0 ,wenn x = 0
//   -1 ,wenn x < 0
//--------------------------------------------------------------
int16_t P_Graphic_sgn(int16_t x)
{
  return (x > 0) ? 1 : (x < 0) ? -1 : 0;
}



//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
int InitTag(void)
{
  BYTE finish=0;
  BYTE id;
  short  llength;
  short  i,j,k;
  short  huftab1,huftab2;
  short  huftabindex;
  BYTE hf_table_index;
  BYTE qt_table_index;
  BYTE comnum;
  unsigned char  *lptemp;
  short  colorount;

  GRAPHIC_JPG.lp=GRAPHIC_JPG.lpJpegBuf+2;

  while (!finish)
  {
    id=*(GRAPHIC_JPG.lp+1);  
    GRAPHIC_JPG.lp+=2; 
    switch (id)
    {
      case M_APP0: 
        llength=MAKEWORD(*(GRAPHIC_JPG.lp+1),*GRAPHIC_JPG.lp);
        GRAPHIC_JPG.lp+=llength;
      break;
      case M_DQT:  
        llength=MAKEWORD(*(GRAPHIC_JPG.lp+1),*GRAPHIC_JPG.lp);  
        qt_table_index=(*(GRAPHIC_JPG.lp+2))&0x0f; 
        lptemp=GRAPHIC_JPG.lp+3;  
        if(llength<80) {
          for(i=0;i<64;i++) {
            GRAPHIC_JPG.qt_table[qt_table_index][i]=(short)*(lptemp++);
          }
        }
        else {
          for(i=0;i<64;i++) {
            GRAPHIC_JPG.qt_table[qt_table_index][i]=(short)*(lptemp++);   
          }
          qt_table_index=(*(lptemp++))&0x0f;
          for(i=0;i<64;i++) {
            GRAPHIC_JPG.qt_table[qt_table_index][i]=(short)*(lptemp++); 
          }
        }
        GRAPHIC_JPG.lp+=llength; 		
      break;
      case M_SOF0: 
        llength=MAKEWORD(*(GRAPHIC_JPG.lp+1),*GRAPHIC_JPG.lp); 
        GRAPHIC_JPG.ImgHeight=MAKEWORD(*(GRAPHIC_JPG.lp+4),*(GRAPHIC_JPG.lp+3)); 
        GRAPHIC_JPG.ImgWidth=MAKEWORD(*(GRAPHIC_JPG.lp+6),*(GRAPHIC_JPG.lp+5));  
        GRAPHIC_JPG.comp_num=*(GRAPHIC_JPG.lp+7); 
        if((GRAPHIC_JPG.comp_num!=1)&&(GRAPHIC_JPG.comp_num!=3))return FUNC_FORMAT_ERROR; 
        if(GRAPHIC_JPG.comp_num==3)	{
          GRAPHIC_JPG.comp_index[0]=*(GRAPHIC_JPG.lp+8); 
          GRAPHIC_JPG.SampRate_Y_H=(*(GRAPHIC_JPG.lp+9))>>4;  
          GRAPHIC_JPG.SampRate_Y_V=(*(GRAPHIC_JPG.lp+9))&0x0f;  
          GRAPHIC_JPG.YQtTable=(short *)GRAPHIC_JPG.qt_table[*(GRAPHIC_JPG.lp+10)];  
          GRAPHIC_JPG.comp_index[1]=*(GRAPHIC_JPG.lp+11); 
          GRAPHIC_JPG.SampRate_U_H=(*(GRAPHIC_JPG.lp+12))>>4; 
          GRAPHIC_JPG.SampRate_U_V=(*(GRAPHIC_JPG.lp+12))&0x0f; 
          GRAPHIC_JPG.UQtTable=(short *)GRAPHIC_JPG.qt_table[*(GRAPHIC_JPG.lp+13)]; 

          GRAPHIC_JPG.comp_index[2]=*(GRAPHIC_JPG.lp+14); 
          GRAPHIC_JPG.SampRate_V_H=(*(GRAPHIC_JPG.lp+15))>>4; 
          GRAPHIC_JPG.SampRate_V_V=(*(GRAPHIC_JPG.lp+15))&0x0f; 
          GRAPHIC_JPG.VQtTable=(short *)GRAPHIC_JPG.qt_table[*(GRAPHIC_JPG.lp+16)]; 
        }
        else {
          GRAPHIC_JPG.comp_index[0]=*(GRAPHIC_JPG.lp+8);
          GRAPHIC_JPG.SampRate_Y_H=(*(GRAPHIC_JPG.lp+9))>>4;
          GRAPHIC_JPG.SampRate_Y_V=(*(GRAPHIC_JPG.lp+9))&0x0f;
          GRAPHIC_JPG.YQtTable=(short *)GRAPHIC_JPG.qt_table[*(GRAPHIC_JPG.lp+10)]; 
          GRAPHIC_JPG.comp_index[1]=*(GRAPHIC_JPG.lp+8);
          GRAPHIC_JPG.SampRate_U_H=1;
          GRAPHIC_JPG.SampRate_U_V=1;
          GRAPHIC_JPG.UQtTable=(short *)GRAPHIC_JPG.qt_table[*(GRAPHIC_JPG.lp+10)];
	  GRAPHIC_JPG.comp_index[2]=*(GRAPHIC_JPG.lp+8);
          GRAPHIC_JPG.SampRate_V_H=1;
          GRAPHIC_JPG.SampRate_V_V=1;
          GRAPHIC_JPG.VQtTable=(short *)GRAPHIC_JPG.qt_table[*(GRAPHIC_JPG.lp+10)];
        }
        GRAPHIC_JPG.lp+=llength;						    
      break;
      case M_DHT:             
        llength=MAKEWORD(*(GRAPHIC_JPG.lp+1),*GRAPHIC_JPG.lp); 
        if (llength<0xd0) {
          huftab1=(short)(*(GRAPHIC_JPG.lp+2))>>4; 
          huftab2=(short)(*(GRAPHIC_JPG.lp+2))&0x0f; 
          huftabindex=huftab1*2+huftab2;  
          lptemp=GRAPHIC_JPG.lp+3;
          for (i=0; i<16; i++) {
            GRAPHIC_JPG.code_len_table[huftabindex][i]=(short)(*(lptemp++));
          }          
          j=0;
          for (i=0; i<16; i++) {
            if(GRAPHIC_JPG.code_len_table[huftabindex][i]!=0) {
              k=0;
              while(k<GRAPHIC_JPG.code_len_table[huftabindex][i]) {
                GRAPHIC_JPG.code_value_table[huftabindex][k+j]=(short)(*(lptemp++));
                k++;
              }
              j+=k;	
            }
          }
          i=0;
          while (GRAPHIC_JPG.code_len_table[huftabindex][i]==0)i++;	   
          for (j=0;j<i;j++) {
            GRAPHIC_JPG.huf_min_value[huftabindex][j]=0;
            GRAPHIC_JPG.huf_max_value[huftabindex][j]=0;
          }
          GRAPHIC_JPG.huf_min_value[huftabindex][i]=0;
          GRAPHIC_JPG.huf_max_value[huftabindex][i]=GRAPHIC_JPG.code_len_table[huftabindex][i]-1;
          for (j=i+1;j<16;j++) {
            GRAPHIC_JPG.huf_min_value[huftabindex][j]=(GRAPHIC_JPG.huf_max_value[huftabindex][j-1]+1)<<1;
            GRAPHIC_JPG.huf_max_value[huftabindex][j]=GRAPHIC_JPG.huf_min_value[huftabindex][j]+GRAPHIC_JPG.code_len_table[huftabindex][j]-1;
          }
          GRAPHIC_JPG.code_pos_table[huftabindex][0]=0;
          for (j=1;j<16;j++) {
            GRAPHIC_JPG.code_pos_table[huftabindex][j]=GRAPHIC_JPG.code_len_table[huftabindex][j-1]+GRAPHIC_JPG.code_pos_table[huftabindex][j-1];
          }
          GRAPHIC_JPG.lp+=llength;
        } 
        else {
          hf_table_index=*(GRAPHIC_JPG.lp+2);
          GRAPHIC_JPG.lp+=2;
          while (hf_table_index!=0xff) {
            huftab1=(short)hf_table_index>>4;
            huftab2=(short)hf_table_index&0x0f;
            huftabindex=huftab1*2+huftab2;
            lptemp=GRAPHIC_JPG.lp+1;
            colorount=0;
            for (i=0; i<16; i++) {
              GRAPHIC_JPG.code_len_table[huftabindex][i]=(short)(*(lptemp++));
              colorount+=GRAPHIC_JPG.code_len_table[huftabindex][i];
            }
            colorount+=17;	
            j=0;
            for (i=0; i<16; i++) {
              if(GRAPHIC_JPG.code_len_table[huftabindex][i]!=0) {
                k=0;
                while(k<GRAPHIC_JPG.code_len_table[huftabindex][i]) {
                  GRAPHIC_JPG.code_value_table[huftabindex][k+j]=(short)(*(lptemp++));
                  k++;
                }
                j+=k;
              }
            }
            i=0;
            while (GRAPHIC_JPG.code_len_table[huftabindex][i]==0)i++;
            for (j=0;j<i;j++) {
              GRAPHIC_JPG.huf_min_value[huftabindex][j]=0;
              GRAPHIC_JPG.huf_max_value[huftabindex][j]=0;
            }
            GRAPHIC_JPG.huf_min_value[huftabindex][i]=0;
            GRAPHIC_JPG.huf_max_value[huftabindex][i]=GRAPHIC_JPG.code_len_table[huftabindex][i]-1;
            for (j=i+1;j<16;j++) {
              GRAPHIC_JPG.huf_min_value[huftabindex][j]=(GRAPHIC_JPG.huf_max_value[huftabindex][j-1]+1)<<1;
              GRAPHIC_JPG.huf_max_value[huftabindex][j]=GRAPHIC_JPG.huf_min_value[huftabindex][j]+GRAPHIC_JPG.code_len_table[huftabindex][j]-1;
            }
            GRAPHIC_JPG.code_pos_table[huftabindex][0]=0;
            for (j=1;j<16;j++) {
              GRAPHIC_JPG.code_pos_table[huftabindex][j]=GRAPHIC_JPG.code_len_table[huftabindex][j-1]+GRAPHIC_JPG.code_pos_table[huftabindex][j-1];
            }
            GRAPHIC_JPG.lp+=colorount;
            hf_table_index=*GRAPHIC_JPG.lp;
          }
        }
      break;
      case M_DRI:
        llength=MAKEWORD(*(GRAPHIC_JPG.lp+1),*GRAPHIC_JPG.lp);
        GRAPHIC_JPG.restart=MAKEWORD(*(GRAPHIC_JPG.lp+3),*(GRAPHIC_JPG.lp+2));
        GRAPHIC_JPG.lp+=llength;
      break;
      case M_SOS:
        llength=MAKEWORD(*(GRAPHIC_JPG.lp+1),*GRAPHIC_JPG.lp);
        comnum=*(GRAPHIC_JPG.lp+2);
        if(comnum!=GRAPHIC_JPG.comp_num) return FUNC_FORMAT_ERROR;
        lptemp=GRAPHIC_JPG.lp+3;
        for (i=0;i<GRAPHIC_JPG.comp_num;i++) {
          if(*lptemp==GRAPHIC_JPG.comp_index[0]) {
            GRAPHIC_JPG.YDcIndex=(*(lptemp+1))>>4;  
            GRAPHIC_JPG.YAcIndex=((*(lptemp+1))&0x0f)+2;
          }
          else {
            GRAPHIC_JPG.UVDcIndex=(*(lptemp+1))>>4;
            GRAPHIC_JPG.UVAcIndex=((*(lptemp+1))&0x0f)+2;
          }
          lptemp+=2;
        }
        GRAPHIC_JPG.lp+=llength;
        finish=1;
      break;
      case M_EOI: 
        return FUNC_FORMAT_ERROR;
      break;
      default:
        if ((id&0xf0)!=0xd0) {
          llength=MAKEWORD(*(GRAPHIC_JPG.lp+1),*GRAPHIC_JPG.lp);
          GRAPHIC_JPG.lp+=llength;
        }
        else {
          GRAPHIC_JPG.lp+=2;
        }
      break;
    }
  }
  return FUNC_OK;
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
void InitTable(void)
{
  short i,j;

  GRAPHIC_JPG.sizei=0;
  GRAPHIC_JPG.sizej=0;
  GRAPHIC_JPG.ImgWidth=0;
  GRAPHIC_JPG.ImgHeight=0;
  GRAPHIC_JPG.rrun=0;
  GRAPHIC_JPG.vvalue=0;
  GRAPHIC_JPG.BitPos=0;
  GRAPHIC_JPG.CurByte=0;
  GRAPHIC_JPG.IntervalFlag=0;
  GRAPHIC_JPG.restart=0;
  for(i=0;i<3;i++) {
    for(j=0;j<64;j++) {
      GRAPHIC_JPG.qt_table[i][j]=0;
    }
  }
  GRAPHIC_JPG.comp_num=0;
  GRAPHIC_JPG.HufTabIndex=0;
  for(i=0;i<3;i++) {
    GRAPHIC_JPG.comp_index[i]=0;
  }
  for(i=0;i<4;i++) {
    for(j=0;j<16;j++) {
      GRAPHIC_JPG.code_len_table[i][j]=0;
      GRAPHIC_JPG.code_pos_table[i][j]=0;
      GRAPHIC_JPG.huf_max_value[i][j]=0;
      GRAPHIC_JPG.huf_min_value[i][j]=0;
    }
  }
  for(i=0;i<4;i++) {
    for(j=0;j<256;j++) {
      GRAPHIC_JPG.code_value_table[i][j]=0;
    }
  }
	
  for(i=0;i<10*64;i++) {
    GRAPHIC_JPG.MCUBuffer[i]=0;
    GRAPHIC_JPG.QtZzMCUBuffer[i]=0;
  }
  for(i=0;i<64;i++) {
    GRAPHIC_JPG.Y[i]=0;
    GRAPHIC_JPG.U[i]=0;
    GRAPHIC_JPG.V[i]=0;
    GRAPHIC_JPG.BlockBuffer[i]=0;
  }
  GRAPHIC_JPG.ycoef=0;
  GRAPHIC_JPG.ucoef=0;
  GRAPHIC_JPG.vcoef=0;
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
int Decode(void)
{
  int funcret;

  GRAPHIC_JPG.Y_in_MCU=GRAPHIC_JPG.SampRate_Y_H*GRAPHIC_JPG.SampRate_Y_V; 
  GRAPHIC_JPG.U_in_MCU=GRAPHIC_JPG.SampRate_U_H*GRAPHIC_JPG.SampRate_U_V; 
  GRAPHIC_JPG.V_in_MCU=GRAPHIC_JPG.SampRate_V_H*GRAPHIC_JPG.SampRate_V_V;  
  GRAPHIC_JPG.H_YtoU=GRAPHIC_JPG.SampRate_Y_H/GRAPHIC_JPG.SampRate_U_H;
  GRAPHIC_JPG.V_YtoU=GRAPHIC_JPG.SampRate_Y_V/GRAPHIC_JPG.SampRate_U_V;
  GRAPHIC_JPG.H_YtoV=GRAPHIC_JPG.SampRate_Y_H/GRAPHIC_JPG.SampRate_V_H;
  GRAPHIC_JPG.V_YtoV=GRAPHIC_JPG.SampRate_Y_V/GRAPHIC_JPG.SampRate_V_V;
  Initialize_Fast_IDCT();
  while((funcret=DecodeMCUBlock())==FUNC_OK) {
    GRAPHIC_JPG.interval++; 
    if((GRAPHIC_JPG.restart)&&(GRAPHIC_JPG.interval % GRAPHIC_JPG.restart==0)) { 
      GRAPHIC_JPG.IntervalFlag=1;  
    }
    else {
      GRAPHIC_JPG.IntervalFlag=0;
    }
    IQtIZzMCUComponent(0);  
    IQtIZzMCUComponent(1);
    IQtIZzMCUComponent(2);
    GetYUV(0); 
    GetYUV(1);
    GetYUV(2);
    StoreBuffer(); 
    GRAPHIC_JPG.sizej+=GRAPHIC_JPG.SampRate_Y_H*8;
    if(GRAPHIC_JPG.sizej>=GRAPHIC_JPG.ImgWidth) {
      GRAPHIC_JPG.sizej=0;
      GRAPHIC_JPG.sizei+=GRAPHIC_JPG.SampRate_Y_V*8;
    }
    if ((GRAPHIC_JPG.sizej==0)&&(GRAPHIC_JPG.sizei>=GRAPHIC_JPG.ImgHeight)) break; 
  }
  return funcret;
}
#endif
   

//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
void GetYUV(short flag)
{
  short	H,VV;
  short	i,j,k,h;
  short *buf;
  short *pQtZzMCU;

  switch(flag)
  {
    case 0:
      H=GRAPHIC_JPG.SampRate_Y_H;
      VV=GRAPHIC_JPG.SampRate_Y_V;
      buf=GRAPHIC_JPG.Y;
      pQtZzMCU=GRAPHIC_JPG.QtZzMCUBuffer;
    break;
    case 1:
      H=GRAPHIC_JPG.SampRate_U_H;
      VV=GRAPHIC_JPG.SampRate_U_V;
      buf=GRAPHIC_JPG.U;
      pQtZzMCU=GRAPHIC_JPG.QtZzMCUBuffer+GRAPHIC_JPG.Y_in_MCU*64;
    break;
    case 2:
      H=GRAPHIC_JPG.SampRate_V_H;
      VV=GRAPHIC_JPG.SampRate_V_V;
      buf=GRAPHIC_JPG.V;
      pQtZzMCU=GRAPHIC_JPG.QtZzMCUBuffer+(GRAPHIC_JPG.Y_in_MCU+GRAPHIC_JPG.U_in_MCU)*64;
    break;
  }
  for (i=0;i<VV;i++) {
    for(j=0;j<H;j++) {
      for(k=0;k<8;k++) {
        for(h=0;h<8;h++) {
          buf[(i*8+k)*GRAPHIC_JPG.SampRate_Y_H*8+j*8+h]=*pQtZzMCU++;
        }
      }
    }
  }
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1 
void StoreBuffer(void)
{
  short i=0,j=0;			    
  unsigned char R,G,B;
  int y,u,v,rr,gg,bb;
  long color;

  for(i=0;i<GRAPHIC_JPG.SampRate_Y_V*8;i++) {
    if((GRAPHIC_JPG.sizei+i)<GRAPHIC_JPG.ImgHeight) {    
      for(j=0;j<GRAPHIC_JPG.SampRate_Y_H*8;j++) {
        if((GRAPHIC_JPG.sizej+j)<GRAPHIC_JPG.ImgWidth) {
          y=GRAPHIC_JPG.Y[i*8*GRAPHIC_JPG.SampRate_Y_H+j];
          u=GRAPHIC_JPG.U[(i/GRAPHIC_JPG.V_YtoU)*8*GRAPHIC_JPG.SampRate_Y_H+j/GRAPHIC_JPG.H_YtoU];
          v=GRAPHIC_JPG.V[(i/GRAPHIC_JPG.V_YtoV)*8*GRAPHIC_JPG.SampRate_Y_H+j/GRAPHIC_JPG.H_YtoV];
          rr=((y<<8)+18*u+367*v)>>8;
          gg=((y<<8)-159*u-220*v)>>8;
          bb=((y<<8)+411*u-29*v)>>8;
          R=(unsigned char)rr;
          G=(unsigned char)gg;
          B=(unsigned char)bb;
          if (rr&0xffffff00) {
            if (rr>255) {
              R=255; 
            }
            else {
              if (rr<0) R=0;
            }
          }
          if (gg&0xffffff00) {
            if (gg>255) {
              G=255; 
            }
            else {
              if (gg<0) G=0;
            }
          }
          if (bb&0xffffff00) {
            if (bb>255) {
              B=255; 
            }
            else {
              if (bb<0) B=0;  
            }
          }
          color=R>>3;
          color=color<<6;
          color |=(G>>2);
          color=color<<5;
          color |=(B>>3);

          if(LCD_DISPLAY_MODE==PORTRAIT) {
            UB_Graphic_DrawPixel(GRAPHIC_JPG.sizej+j+GRAPHIC_JPG.CurX,GRAPHIC_JPG.sizei+i+GRAPHIC_JPG.CurY,color);
          }
          else {
        	UB_Graphic_DrawPixel(GRAPHIC_JPG.ImgHeight-GRAPHIC_JPG.sizei-i+GRAPHIC_JPG.CurX-1,GRAPHIC_JPG.sizej+j+GRAPHIC_JPG.CurY,color);
          }
        }
        else  break;
      } 	    
    }
    else break;
  }
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
int DecodeMCUBlock(void)
{
  short *lpMCUBuffer;
  short i,j;
  int funcret;

  if (GRAPHIC_JPG.IntervalFlag==1) {
    GRAPHIC_JPG.lp+=2;
    GRAPHIC_JPG.ycoef=0;
    GRAPHIC_JPG.ucoef=0;
    GRAPHIC_JPG.vcoef=0;
    GRAPHIC_JPG.BitPos=0;
    GRAPHIC_JPG.CurByte=0;
  }
  switch(GRAPHIC_JPG.comp_num)
  {
    case 3:
      lpMCUBuffer=GRAPHIC_JPG.MCUBuffer;
      for (i=0;i<GRAPHIC_JPG.SampRate_Y_H*GRAPHIC_JPG.SampRate_Y_V;i++) {
        funcret=HufBlock(GRAPHIC_JPG.YDcIndex,GRAPHIC_JPG.YAcIndex); 
        if (funcret!=FUNC_OK) return funcret;
        GRAPHIC_JPG.BlockBuffer[0]=GRAPHIC_JPG.BlockBuffer[0]+GRAPHIC_JPG.ycoef; 
        GRAPHIC_JPG.ycoef=GRAPHIC_JPG.BlockBuffer[0];
        for (j=0;j<64;j++) {
          *lpMCUBuffer++=GRAPHIC_JPG.BlockBuffer[j];
        }
      }
      for (i=0;i<GRAPHIC_JPG.SampRate_U_H*GRAPHIC_JPG.SampRate_U_V;i++) {
        funcret=HufBlock(GRAPHIC_JPG.UVDcIndex,GRAPHIC_JPG.UVAcIndex);
        if (funcret!=FUNC_OK) return funcret;
        GRAPHIC_JPG.BlockBuffer[0]=GRAPHIC_JPG.BlockBuffer[0]+GRAPHIC_JPG.ucoef;
        GRAPHIC_JPG.ucoef=GRAPHIC_JPG.BlockBuffer[0];
        for (j=0;j<64;j++) {
          *lpMCUBuffer++=GRAPHIC_JPG.BlockBuffer[j];
        }
      }
      for (i=0;i<GRAPHIC_JPG.SampRate_V_H*GRAPHIC_JPG.SampRate_V_V;i++) {
        funcret=HufBlock(GRAPHIC_JPG.UVDcIndex,GRAPHIC_JPG.UVAcIndex);
        if (funcret!=FUNC_OK) return funcret;
        GRAPHIC_JPG.BlockBuffer[0]=GRAPHIC_JPG.BlockBuffer[0]+GRAPHIC_JPG.vcoef;
        GRAPHIC_JPG.vcoef=GRAPHIC_JPG.BlockBuffer[0];
        for (j=0;j<64;j++) {
          *lpMCUBuffer++=GRAPHIC_JPG.BlockBuffer[j];
        }
      }
    break;
    case 1:	
      lpMCUBuffer=GRAPHIC_JPG.MCUBuffer;
      funcret=HufBlock(GRAPHIC_JPG.YDcIndex,GRAPHIC_JPG.YAcIndex);
      if (funcret!=FUNC_OK) return funcret;
      GRAPHIC_JPG.BlockBuffer[0]=GRAPHIC_JPG.BlockBuffer[0]+GRAPHIC_JPG.ycoef;
      GRAPHIC_JPG.ycoef=GRAPHIC_JPG.BlockBuffer[0];
      for (j=0;j<64;j++) {
        *lpMCUBuffer++=GRAPHIC_JPG.BlockBuffer[j];
      }
      for (i=0;i<128;i++) {
        *lpMCUBuffer++=0;
      }
    break;
    default:
      return FUNC_FORMAT_ERROR;
  }
  return FUNC_OK;
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
int HufBlock(BYTE dchufindex,BYTE achufindex)
{
  short count=0;
  short i;
  int funcret;

  GRAPHIC_JPG.HufTabIndex=dchufindex;
  funcret=DecodeElement();
  if(funcret!=FUNC_OK) return funcret;

  GRAPHIC_JPG.BlockBuffer[count++]=GRAPHIC_JPG.vvalue;
  GRAPHIC_JPG.HufTabIndex=achufindex;
  while (count<64) {
    funcret=DecodeElement();
    if(funcret!=FUNC_OK) return funcret;
    if ((GRAPHIC_JPG.rrun==0)&&(GRAPHIC_JPG.vvalue==0)) {
      for (i=count;i<64;i++) {
        GRAPHIC_JPG.BlockBuffer[i]=0;
      }
      count=64;
    }
    else {
      for (i=0;i<GRAPHIC_JPG.rrun;i++) {
        GRAPHIC_JPG.BlockBuffer[count++]=0;
      }
      GRAPHIC_JPG.BlockBuffer[count++]=GRAPHIC_JPG.vvalue;
    }
  }
  return FUNC_OK;
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
int DecodeElement(void)
{
  int thiscode,tempcode;
  unsigned short temp,valueex;
  short codelen;
  BYTE hufexbyte,runsize,tempsize,sign;
  BYTE newbyte,lastbyte;

  if(GRAPHIC_JPG.BitPos>=1) {
    GRAPHIC_JPG.BitPos--;
    thiscode=(BYTE)GRAPHIC_JPG.CurByte>>GRAPHIC_JPG.BitPos;
    GRAPHIC_JPG.CurByte=GRAPHIC_JPG.CurByte&GRAPHIC_JPG_AND[GRAPHIC_JPG.BitPos];
  }
  else {
    lastbyte=ReadByte();
    GRAPHIC_JPG.BitPos--;
    newbyte=GRAPHIC_JPG.CurByte&GRAPHIC_JPG_AND[GRAPHIC_JPG.BitPos];
    thiscode=lastbyte>>7;
    GRAPHIC_JPG.CurByte=newbyte;
  }
  codelen=1;
	 
  while ((thiscode<GRAPHIC_JPG.huf_min_value[GRAPHIC_JPG.HufTabIndex][codelen-1])||
         (GRAPHIC_JPG.code_len_table[GRAPHIC_JPG.HufTabIndex][codelen-1]==0)||
         (thiscode>GRAPHIC_JPG.huf_max_value[GRAPHIC_JPG.HufTabIndex][codelen-1])) {
    if(GRAPHIC_JPG.BitPos>=1) {
      GRAPHIC_JPG.BitPos--;
      tempcode=(BYTE)GRAPHIC_JPG.CurByte>>GRAPHIC_JPG.BitPos;
      GRAPHIC_JPG.CurByte=GRAPHIC_JPG.CurByte&GRAPHIC_JPG_AND[GRAPHIC_JPG.BitPos];
    }
    else {
      lastbyte=ReadByte();
      GRAPHIC_JPG.BitPos--;
      newbyte=GRAPHIC_JPG.CurByte&GRAPHIC_JPG_AND[GRAPHIC_JPG.BitPos];
      tempcode=(BYTE)lastbyte>>7;
      GRAPHIC_JPG.CurByte=newbyte;
    }
    thiscode=(thiscode<<1)+tempcode;
    codelen++;
    if(codelen>16)return FUNC_FORMAT_ERROR;	   
  } 
  temp=thiscode-GRAPHIC_JPG.huf_min_value[GRAPHIC_JPG.HufTabIndex][codelen-1]+GRAPHIC_JPG.code_pos_table[GRAPHIC_JPG.HufTabIndex][codelen-1];
  hufexbyte=(BYTE)GRAPHIC_JPG.code_value_table[GRAPHIC_JPG.HufTabIndex][temp];
  GRAPHIC_JPG.rrun=(short)(hufexbyte>>4);
  runsize=hufexbyte&0x0f;
  if(runsize==0) {
    GRAPHIC_JPG.vvalue=0;
    return FUNC_OK;
  }
  tempsize=runsize;
  if(GRAPHIC_JPG.BitPos>=runsize) {
    GRAPHIC_JPG.BitPos-=runsize;
    valueex=(BYTE)GRAPHIC_JPG.CurByte>>GRAPHIC_JPG.BitPos;
    GRAPHIC_JPG.CurByte=GRAPHIC_JPG.CurByte&GRAPHIC_JPG_AND[GRAPHIC_JPG.BitPos];
  }
  else {
    valueex=GRAPHIC_JPG.CurByte;
    tempsize-=GRAPHIC_JPG.BitPos;
    while(tempsize>8) {
      lastbyte=ReadByte();
      valueex=(valueex<<8)+(BYTE)lastbyte;
      tempsize-=8;
    } 
    lastbyte=ReadByte();
    GRAPHIC_JPG.BitPos-=tempsize;
    valueex=(valueex<<tempsize)+(lastbyte>>GRAPHIC_JPG.BitPos);
    GRAPHIC_JPG.CurByte=lastbyte&GRAPHIC_JPG_AND[GRAPHIC_JPG.BitPos];
  }  
  sign=valueex>>(runsize-1);
  if(sign) {
    GRAPHIC_JPG.vvalue=valueex;
  }
  else {
    valueex=valueex^0xffff;
    temp=0xffff<<runsize;
    GRAPHIC_JPG.vvalue=-(short)(valueex^temp);
  }
  return FUNC_OK;
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
void IQtIZzMCUComponent(short flag)
{
  short H,VV;
  short i,j;
  short *pQtZzMCUBuffer;
  short  *pMCUBuffer;

  switch(flag){
    case 0:
      H=GRAPHIC_JPG.SampRate_Y_H;
      VV=GRAPHIC_JPG.SampRate_Y_V;
      pMCUBuffer=GRAPHIC_JPG.MCUBuffer;
      pQtZzMCUBuffer=GRAPHIC_JPG.QtZzMCUBuffer;
    break;
    case 1:
      H=GRAPHIC_JPG.SampRate_U_H;
      VV=GRAPHIC_JPG.SampRate_U_V;
      pMCUBuffer=GRAPHIC_JPG.MCUBuffer+GRAPHIC_JPG.Y_in_MCU*64;
      pQtZzMCUBuffer=GRAPHIC_JPG.QtZzMCUBuffer+GRAPHIC_JPG.Y_in_MCU*64;
    break;
    case 2:
      H=GRAPHIC_JPG.SampRate_V_H;
      VV=GRAPHIC_JPG.SampRate_V_V;
      pMCUBuffer=GRAPHIC_JPG.MCUBuffer+(GRAPHIC_JPG.Y_in_MCU+GRAPHIC_JPG.U_in_MCU)*64;
      pQtZzMCUBuffer=GRAPHIC_JPG.QtZzMCUBuffer+(GRAPHIC_JPG.Y_in_MCU+GRAPHIC_JPG.U_in_MCU)*64;
    break;
  }
  for(i=0;i<VV;i++) {
    for (j=0;j<H;j++) {
      IQtIZzBlock(pMCUBuffer+(i*H+j)*64,pQtZzMCUBuffer+(i*H+j)*64,flag);
    }
  }
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
void IQtIZzBlock(short  *s ,short * d,short flag)
{
  short i,j;
  short tag;
  short *pQt;
  int buffer2[8][8];
  int *buffer1;
  short offset;

  switch(flag)
  {
    case 0:
      pQt=GRAPHIC_JPG.YQtTable;
      offset=128;
    break;
    case 1:
      pQt=GRAPHIC_JPG.UQtTable;
      offset=0;
    break;
    case 2:
      pQt=GRAPHIC_JPG.VQtTable;
      offset=0;
    break;
  }

  for(i=0;i<8;i++) {
    for(j=0;j<8;j++) {
      tag=GRAPHIC_JPG_ZZ[i][j];
      buffer2[i][j]=(int)s[tag]*(int)pQt[tag];
    }
  }
    
  buffer1=(int *)buffer2;
  Fast_IDCT(buffer1); 
  for(i=0;i<8;i++) {
    for(j=0;j<8;j++) {
      d[i*8+j]=buffer2[i][j]+offset;
    }
  }
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
void Fast_IDCT(int * block)
{
  short i;
		 
  for (i=0; i<8; i++)idctrow(block+8*i); 
  for (i=0; i<8; i++)idctcol(block+i);    
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
BYTE ReadByte(void)
{
  BYTE  i;

  i=*(GRAPHIC_JPG.lp++);
  if(i==0xff)GRAPHIC_JPG.lp++;   
  GRAPHIC_JPG.BitPos=8;
  GRAPHIC_JPG.CurByte=i;
  return i;
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
void Initialize_Fast_IDCT(void)
{
  short i;

  GRAPHIC_JPG.iclp = GRAPHIC_JPG.iclip+512;
  for (i= -512; i<512; i++) {
    GRAPHIC_JPG.iclp[i] = (i<-256) ? -256 : ((i>255) ? 255 : i);
  }
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
void idctrow(int * blk)
{
  int x0, x1, x2, x3, x4, x5, x6, x7, x8;
	
  if (!((x1 = blk[4]<<11) | (x2 = blk[6]) | (x3 = blk[2]) |
     (x4 = blk[1]) | (x5 = blk[7]) | (x6 = blk[5]) | (x7 = blk[3]))) {
    blk[0]=blk[1]=blk[2]=blk[3]=blk[4]=blk[5]=blk[6]=blk[7]=blk[0]<<3;
    return;
  }
  
  x0 = (blk[0]<<11) + 128;
  
  x8 = W7*(x4+x5);
  x4 = x8 + (W1-W7)*x4;
  x5 = x8 - (W1+W7)*x5;
  x8 = W3*(x6+x7);
  x6 = x8 - (W3-W5)*x6;
  x7 = x8 - (W3+W5)*x7;

  x8 = x0 + x1;
  x0 -= x1;
  x1 = W6*(x3+x2);
  x2 = x1 - (W2+W6)*x2;
  x3 = x1 + (W2-W6)*x3;
  x1 = x4 + x6;
  x4 -= x6;
  x6 = x5 + x7;
  x5 -= x7;

  x7 = x8 + x3;
  x8 -= x3;
  x3 = x0 + x2;
  x0 -= x2;
  x2 = (181*(x4+x5)+128)>>8;
  x4 = (181*(x4-x5)+128)>>8;

  blk[0] = (x7+x1)>>8;
  blk[1] = (x3+x2)>>8;
  blk[2] = (x0+x4)>>8;
  blk[3] = (x8+x6)>>8;
  blk[4] = (x8-x6)>>8;
  blk[5] = (x0-x4)>>8;
  blk[6] = (x3-x2)>>8;
  blk[7] = (x7-x1)>>8;
}
#endif


//--------------------------------------------------------------
#if GRAPHIC_USE_JPG==1
void idctcol(int * blk)
{
  int x0, x1, x2, x3, x4, x5, x6, x7, x8;
  
  if (!((x1 = (blk[8*4]<<8)) | (x2 = blk[8*6]) | (x3 = blk[8*2]) |
    (x4 = blk[8*1]) | (x5 = blk[8*7]) | (x6 = blk[8*5]) | (x7 = blk[8*3]))) {
    blk[8*0]=blk[8*1]=blk[8*2]=blk[8*3]=blk[8*4]=blk[8*5]=blk[8*6]=blk[8*7]=GRAPHIC_JPG.iclp[(blk[8*0]+32)>>6];
    return;
  }
  x0 = (blk[8*0]<<8) + 8192;

  x8 = W7*(x4+x5) + 4;
  x4 = (x8+(W1-W7)*x4)>>3;
  x5 = (x8-(W1+W7)*x5)>>3;
  x8 = W3*(x6+x7) + 4;
  x6 = (x8-(W3-W5)*x6)>>3;
  x7 = (x8-(W3+W5)*x7)>>3;

  x8 = x0 + x1;
  x0 -= x1;
  x1 = W6*(x3+x2) + 4;
  x2 = (x1-(W2+W6)*x2)>>3;
  x3 = (x1+(W2-W6)*x3)>>3;
  x1 = x4 + x6;
  x4 -= x6;
  x6 = x5 + x7;
  x5 -= x7;

  x7 = x8 + x3;
  x8 -= x3;
  x3 = x0 + x2;
  x0 -= x2;
  x2 = (181*(x4+x5)+128)>>8;
  x4 = (181*(x4-x5)+128)>>8;

  blk[8*0] = GRAPHIC_JPG.iclp[(x7+x1)>>14];
  blk[8*1] = GRAPHIC_JPG.iclp[(x3+x2)>>14];
  blk[8*2] = GRAPHIC_JPG.iclp[(x0+x4)>>14];
  blk[8*3] = GRAPHIC_JPG.iclp[(x8+x6)>>14];
  blk[8*4] = GRAPHIC_JPG.iclp[(x8-x6)>>14];
  blk[8*5] = GRAPHIC_JPG.iclp[(x0-x4)>>14];
  blk[8*6] = GRAPHIC_JPG.iclp[(x3-x2)>>14];
  blk[8*7] = GRAPHIC_JPG.iclp[(x7-x1)>>14];
}
#endif



