//--------------------------------------------------------------
// File     : stm32_ub_touch_ads7843.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_TOUCH_ADS7843_H
#define __STM32F4_UB_TOUCH_ADS7843_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_tim.h"
#include "misc.h"
#include <stdbool.h>


// Touch-Status
typedef enum {
  TOUCH_PRESSED  = 0,   // Touch ist betaetigt
  TOUCH_RELEASED = 1	// Touch ist nicht betaetigt
}Touch_Status_t;

// Pixelkoordinaten
typedef struct {
  uint16_t xp;
  uint16_t yp;
}Touch_Pixel_t;

// Konstantenwerte zum Umrechnen der Rohdaten in Display-Daten
typedef struct {
  float A;
  float B;
  float C;
  float D;
  float E;
  float F;
  uint16_t xp1;
  uint16_t yp1;
  uint16_t xp2;
  uint16_t yp2;
  uint16_t xp3;
  uint16_t yp3;
}Touch_Konstanten_t;


//-----------------------------------------
// interne Struktur fr die Kalibration
//-----------------------------------------
typedef struct {
  uint32_t timer_cnt;      // Counter vom Timer
  Touch_Pixel_t raw;       // Rohdaten der Pixelkoordinaten
  Touch_Pixel_t temp;      // Temp Pixelwerte
  Touch_Konstanten_t wert; // Konstantenwerte
}Touch_Kal_t;
Touch_Kal_t Touch_Kal;


//-----------------------------------------
// Globale Struktur der Touch-Daten
//-----------------------------------------
typedef struct {
  Touch_Status_t status;   // Status (gedrckt, losgelassen)
  Touch_Pixel_t pos;       // Display Pixelkoordinate
}Touch_Data_t;
Touch_Data_t Touch_Data;


//--------------------------------------------------------------
// Kommandos an den Touch-Controller
// Lnge = 24Bit
// MSB = CMD-Byte
//--------------------------------------------------------------
#define  UB_TOUCH_CMD_CH3  0x900000   // CH=X+, Mode=12bit, DiffEnded, PwrDown
#define  UB_TOUCH_CMD_CH4  0xD00000   // CH=Y+, Mode=12bit, DiffEnded, PwrDown


//--------------------------------------------------------------
// Defines
// Hinweis : die Pausenzeiten sind nicht per Timer gemacht
//           Vorsicht bei anderen Optimierungsstufen als -O0
//--------------------------------------------------------------
#define  UB_TOUCH_MESS_DELAY   100    // Pause bis Messung startet
#define  UB_TOUCH_CLK_DELAY    1000   // pause fr das SPI-Clock-Signal


//--------------------------------------------------------------
// mit MAX_DIFF wird festgelegt wie weit zwei benachbarte Pixel
// maximal entfernt sein drfen um als gtlig erkannt zu werden
//--------------------------------------------------------------
#define  UB_TOUCH_MAX_DIFF     30     // max Pixel differenz


//--------------------------------------------------------------
// Touch Kalibrationspunkte
//--------------------------------------------------------------
#define   UB_TOUCH_KAL_XP1     120     // 50% von MAXX
#define   UB_TOUCH_KAL_YP1     288     // 90% von MAXY
#define   UB_TOUCH_KAL_XP2      24     // 10% von MAXX
#define   UB_TOUCH_KAL_YP2     160     // 50% von MAXY
#define   UB_TOUCH_KAL_XP3     216     // 90% von MAXX
#define   UB_TOUCH_KAL_YP3      32     // 10% von MAXY

#define   UB_TOUCH_KAL_CBG    RGB_COL_BLACK  // Hintergrundfarbe bei Kalibration
#define   UB_TOUCH_KAL_CPKT   RGB_COL_WHITE  // Farbe vom Zielkreuz
#define   UB_TOUCH_KAL_COK    RGB_COL_GREEN  // Hintergrund wenn OK
#define   UB_TOUCH_KAL_CERR   RGB_COL_RED    // Hintergrund wenn Error

#define   UB_TOUCH_KAL_DELAY   10     // Pause beim kalibrieren (10 = 500ms)

//--------------------------------------------------------------
#define   UB_TOUCH_DC_TIME     10     // DoubleClick-Zeit (10=500ms)
#define   UB_TOUCH_DC_TIMEOUT  20     // DoubleClick-Timeout (20=1sec)



//--------------------------------------------------------------
// Timer fr Touch => TIM5
// Grundfrequenz = 2*APB1 (APB1=42MHz) => TIM_CLK=84MHz
// TIM_Frq = TIM_CLK/(periode+1)/(vorteiler+1)
// TIM_Frq = 20Hz => 50ms (nicht kleiner als 30ms einstellen !!)
//--------------------------------------------------------------
#define   UB_TOUCH_TIM              TIM5
#define   UB_TOUCH_TIM_CLK          RCC_APB1Periph_TIM5
#define   UB_TOUCH_TIM_PERIODE      4999
#define   UB_TOUCH_TIM_PRESCALE     839
#define   UB_TOUCH_TIM_IRQ          TIM5_IRQn
#define   UB_TOUCH_TIM_ISR_HANDLER  TIM5_IRQHandler



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
ErrorStatus UB_Touch_Init(void);
ErrorStatus UB_Toch_Calibrate(void);
bool UB_Touch_OnPressed(void);
bool UB_Touch_OnClick(void);
bool UB_Touch_OnRelease(void);
bool UB_Touch_OnDoubleClick(void);

//--------------------------------------------------------------
#endif // __STM32F4_UB_TOUCH_ADS7843_H
